@@ -1,34 +1,60 @@
 # -*- coding: utf-8 -*-
-"""base -- base behaviors for Owyl.
+"""core -- core behaviors for Owyl.
 
 Copyright 2008 David Eyk. All rights reserved.
 
 $Author: david.eyk $\n
-$Rev: 20 $\n
-$Date: 2009-01-05 22:43:30 -0600 (Mon, 05 Jan 2009) $
+$Rev: 30 $\n
+$Date: 2009-01-08 07:39:11 -0600 (Thu, 08 Jan 2009) $
 """
 
 __author__ = "$Author: david.eyk $"[9:-2]
-__revision__ = "$Rev: 20 $"[6:-2]
-__date__ = "$Date: 2009-01-05 22:43:30 -0600 (Mon, 05 Jan 2009) $"[7:-2]
+__revision__ = "$Rev: 30 $"[6:-2]
+__date__ = "$Date: 2009-01-08 07:39:11 -0600 (Thu, 08 Jan 2009) $"[7:-2]
+
+import sys
 
 try:
     from mx.Stack import Stack, EmptyError
 except ImportError:
-    from .stack import Stack, EmptyError
+    from stack import Stack, EmptyError
 
 RETURN_VALUES = (True, False, None)
 
+__all__ = ['task', 'visit', 'succeed', 'fail', 'succeedAfter', 'failAfter',
+           'sequence', 'selector', 'parallel', 'PARALLEL_SUCCESS',
+           'throw', 'catch']
+
 def task(func):
     """Task decorator.
 
     Decorate a generator function to produce a re-usable generator
     factory for the given task.
     """
-    def initTask(*args, **initkwargs):
+    def initTask(**initkwargs):
         def makeIterator(**runkwargs):
             runkwargs.update(initkwargs)
-            iterator = func(*args, **runkwargs)
+            iterator = func(**runkwargs)
+            return iterator
+        makeIterator.__name__ = func.__name__
+        makeIterator.__doc__ = func.__doc__
+        return makeIterator
+    initTask.__doc__ = func.__doc__
+    initTask.__name__ = func.__name__
+    return initTask
+
+def parent_task(func):
+    """Parent task decorator.
+
+    A parent task is a task that accepts children.
+
+    Decorate a generator function to produce a re-usable generator
+    factory for the given task.
+    """
+    def initTask(*children, **initkwargs):
+        def makeIterator(**runkwargs):
+            runkwargs.update(initkwargs)
+            iterator = func(*children, **runkwargs)
             return iterator
         makeIterator.__name__ = func.__name__
         makeIterator.__doc__ = func.__doc__
@@ -82,7 +108,7 @@ def visit(tree, **kwargs):
                 s.push((current, cur_name))
                 current, cur_name = (child(**kwargs), child.__name__)
                 #print "Descending into %s" % (cur_name,)
-                yield None
+                
         except StopIteration:
             try:
                 current, cur_name = s.pop()
@@ -91,16 +117,15 @@ def visit(tree, **kwargs):
             except EmptyError:
                 raise StopIteration
         except Exception, e:
-            exc = e.__class__(*e.args)
+            exc = e.__class__
             try:
                 # Give the parent task a chance to handle the exception.
                 current, cur_name = s.pop()
-                current.throw(exc)
+                current.throw(exc, e.message, sys.exc_info()[2])()
             except EmptyError:
                 # Give up if the exception has propagated all the way
                 # up the tree:
-                raise exc
-
+                raise e
         
 @task
 def succeed(**kwargs):
@@ -115,7 +140,36 @@ def fail(**kwargs):
     yield False
 
 @task
-def sequence(*args, **kwargs):
+def succeedAfter(**kwargs):
+    """Succeed after a given number of iterations.
+
+    Yields 'None' 'after' times.
+
+    @keyword after: How many iterations to succeed after.
+    @type after: int
+    """
+    after = kwargs.pop('after', 1)
+    for x in xrange(after):
+        yield None
+    yield True
+
+@task
+def failAfter(**kwargs):
+    """Fail after a given number of iterations.
+
+    Yields 'None' 'after' times.
+
+    @keyword after: How many iterations to fail after.
+    @type after: int
+    """
+    after = kwargs.pop('after', 1)
+    for x in xrange(after):
+        yield None
+    yield False
+
+
+@parent_task
+def sequence(*children, **kwargs):
     """Run tasks in sequence until one fails.
 
     The sequence will run each task in sequence until one fails,
@@ -123,18 +177,20 @@ def sequence(*args, **kwargs):
 
     For more information, see the discussion at
     U{http://aigamedev.com/hierarchical-logic/sequence}.
+
+    @param children: tasks to run in sequence as children.
     """
     final_value = True
-    for child in args:
-        result = (yield child)
-        if not result:
+    for child in children:
+        result = yield child
+        if not result and result is not None:
             final_value = False
             break
-        yield None
+        
     yield final_value
 
-@task
-def selector(*args, **kwargs):
+@parent_task
+def selector(*children, **kwargs):
     """Run tasks in sequence until one succeeds.
 
     The selector will run each task in sequence until one succeeds,
@@ -143,15 +199,15 @@ def selector(*args, **kwargs):
     For more information, see the discussion at
     U{http://aigamedev.com/hierarchical-logic/selector}.
 
-    @param *args: tasks to run in sequence as children.
+    @param children: child tasks to select from.
     """
     final_value = False
-    for child in args:
+    for child in children:
         result = (yield child)
         if result:
             final_value = True
             break
-        yield None
+        
     yield final_value
 
 
@@ -172,8 +228,8 @@ class PARALLEL_SUCCESS(Enum):
     REQUIRE_ALL = "ALL"
     REQUIRE_ONE = "ONE"
 
-@task
-def parallel(*args, **kwargs):
+@parent_task
+def parallel(*children, **kwargs):
     """Run tasks in parallel until the success policy is fulfilled or broken.
 
     If the success policy is met, return a success. If the policy is
@@ -182,9 +238,9 @@ def parallel(*args, **kwargs):
     For more information, see the discussion at
     U{aigamedev.com/hierarchical-logic/parallel}.
 
-    @param *args: tasks to run in parallel as children.
+    @param children: tasks to run in parallel as children.
 
-    @param policy: The success policy. All must succeed, 
+    @keyword policy: The success policy. All must succeed, 
                    or only one must succeed.
     @type policy: C{PARALLEL_SUCCESS.REQUIRE_ALL} or 
                   C{PARALLEL_SUCCESS.REQUIRE_ONE}.
@@ -192,7 +248,7 @@ def parallel(*args, **kwargs):
     return_values = (True, False)
     policy = kwargs.pop('policy', PARALLEL_SUCCESS.REQUIRE_ONE)
     all_must_succeed = (policy == PARALLEL_SUCCESS.REQUIRE_ALL)
-    visits = [visit(arg, **kwargs) for arg in args]
+    visits = [visit(arg, **kwargs) for arg in children]
     final_value = True
     while True:
         try:
@@ -208,20 +264,22 @@ def parallel(*args, **kwargs):
                         break
                     else:
                         final_value = result
-                yield None
+                
         except StopIteration:
             break
+        except EmptyError:
+            break
     yield final_value
 
 @task
 def throw(**kwargs):
     """Throw (raise) an exception.
 
-    @param throws: An Exception to throw.
+    @keyword throws: An Exception to throw.
     @type throws: C{Exception}
 
-    @param throws_message: Text to instantiate C{throws} with.
-    @param throws_message: C{str}
+    @keyword throws_message: Text to instantiate C{throws} with.
+    @type throws_message: C{str}
     """
     throws = kwargs.pop('throws', Exception)
     throws_message = kwargs.pop('throws_message', '')
@@ -230,4 +288,27 @@ def throw(**kwargs):
             return self
         def next(self):
             raise throws(throws_message)
-    return gen
+    return gen()
+
+@parent_task
+def catch(child, **kwargs):
+    """Catch a raised exception from child and run an alternate branch.
+
+    Note: this will not catch exceptions raised in the branch.
+
+    @keyword caught: An Exception to catch.
+    @type caught: C{Exception}
+
+    @keyword branch: An alternate tree to visit when caught.
+    """
+    caught = kwargs.pop('caught', Exception)
+    branch = kwargs.pop('branch', fail())
+    
+    result = None
+    try:
+        while result is None:
+            result = (yield child)
+    except caught, e:
+        while result is None:
+            result = (yield branch)
+    yield result
